package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import cn.com.infosec.crypto.params.RSAPrivateCrtKeyParameters;
import java.math.BigInteger;

class RSACoreEngine {
   private RSAKeyParameters key;
   private boolean forEncryption;

   public void init(boolean forEncryption, CipherParameters param) {
      if (param instanceof ParametersWithRandom) {
         this.key = (RSAKeyParameters)((ParametersWithRandom)param).getParameters();
      } else {
         this.key = (RSAKeyParameters)param;
      }

      this.forEncryption = forEncryption;
   }

   public int getInputBlockSize() {
      RSACoreEngine var10000 = this;
      int var1 = this.key.getModulus().bitLength();
      return var10000.forEncryption ? (var1 + 7) / 8 - 1 : (var1 + 7) / 8;
   }

   public int getOutputBlockSize() {
      RSACoreEngine var10000 = this;
      int var1 = this.key.getModulus().bitLength();
      return var10000.forEncryption ? (var1 + 7) / 8 : (var1 + 7) / 8 - 1;
   }

   public BigInteger convertInput(byte[] in, int inOff, int inLen) {
      if (inLen <= this.getInputBlockSize() + 1) {
         if (inLen == this.getInputBlockSize() + 1 && !this.forEncryption) {
            throw new DataLengthException("input too large for RSA cipher.");
         } else {
            if (inOff != 0 || inLen != in.length) {
               byte[] var10000 = in;
               in = new byte[inLen];
               System.arraycopy(var10000, inOff, in, 0, inLen);
            }

            BigInteger inOff1;
            BigInteger var5 = inOff1 = new BigInteger;
            inOff1.<init>(1, in);
            if (var5.compareTo(this.key.getModulus()) < 0) {
               return inOff1;
            } else {
               throw new DataLengthException("input too large for RSA cipher.");
            }
         }
      } else {
         throw new DataLengthException("input too large for RSA cipher.");
      }
   }

   public byte[] convertOutput(BigInteger result) {
      byte[] result1 = result.toByteArray();
      byte[] var10000;
      byte[] var2;
      int var3;
      if (this.forEncryption) {
         if (result1[0] == 0 && result1.length > this.getOutputBlockSize()) {
            var10000 = var2 = new byte[var3 = result1.length - 1];
            System.arraycopy(result1, 1, var2, 0, var3);
            return var10000;
         }

         if (result1.length < this.getOutputBlockSize()) {
            var10000 = var2 = new byte[var3 = this.getOutputBlockSize()];
            byte[] var10001 = result1;
            var3 -= result1.length;
            int result2 = result1.length;
            System.arraycopy(var10001, 0, var2, var3, result2);
            return var10000;
         }
      } else if (result1[0] == 0) {
         var10000 = var2 = new byte[var3 = result1.length - 1];
         System.arraycopy(result1, 1, var2, 0, var3);
         return var10000;
      }

      return result1;
   }

   public BigInteger processBlock(BigInteger input) {
      RSAKeyParameters var2;
      if ((var2 = this.key) instanceof RSAPrivateCrtKeyParameters) {
         RSAPrivateCrtKeyParameters var10001 = (RSAPrivateCrtKeyParameters)var2;
         RSAPrivateCrtKeyParameters var10002 = (RSAPrivateCrtKeyParameters)var2;
         RSAPrivateCrtKeyParameters var10003 = (RSAPrivateCrtKeyParameters)var2;
         BigInteger var6 = ((RSAPrivateCrtKeyParameters)var2).getP();
         BigInteger var7 = var10002.getQ();
         BigInteger var3 = var10003.getDP();
         BigInteger var4 = var10002.getDQ();
         BigInteger var5 = var10001.getQInv();
         return input.remainder(var6).modPow(var3, var6).subtract(input = input.remainder(var7).modPow(var4, var7)).multiply(var5).mod(var6).multiply(var7).add(input);
      } else {
         return input.modPow(var2.getExponent(), this.key.getModulus());
      }
   }
}
